@@ -11,14 +11,15 @@ package Image::ExifTool::HtmlDump;
 use strict;
 use vars qw($VERSION);
 use Image::ExifTool;    # only for FinishTiffDump()
+use Image::ExifTool::HTML qw(EscapeHTML);
 
-$VERSION = '1.21';
+$VERSION = '1.30';
 
 sub DumpTable($$$;$$$$$);
 sub Open($$$;@);
 sub Write($@);
 
-my ($bkgStart, $bkgEnd, $bkgSpan);
+my ($bkgStart, $bkgEnd, @bkgSpan);
 
 my $htmlHeader1 = <<_END_PART_1_;
 <!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0 Transitional//EN"
@@ -43,6 +44,28 @@ my $htmlHeader2 = <<_END_PART_2_;
 .H { color: #0000ff } /* highlighted tag name */
 .F { color: #aa00dd } /* actual offset differs */
 .M { text-decoration: underline } /* maker notes data */
+.tt { /* tooltip text */
+    visibility: hidden;
+    position: absolute;
+    white-space: nowrap;
+    top: 0;
+    left: 0;
+    font-family: Verdana, sans-serif;
+    font-size: .7em;
+    padding: 2px 4px;
+    border: 1px solid gray;
+    z-index: 3;
+}
+.tb { /* tooltip background */
+    visibility: hidden;
+    position: absolute;
+    background: #ffffdd;
+    opacity: 0.8;
+    -moz-opacity: 0.8;
+    filter: alpha(opacity=80);
+    -ms-filter: 'progid:DXImageTransform.Microsoft.Alpha(Opacity=80)';
+    z-index: 2;
+}
 /* table styles */
 table.dump {
   border-top: 1px solid gray;
@@ -60,8 +83,19 @@ body  { color: black; background: white }
 </style>
 <script language="JavaScript" type="text/JavaScript">
 <!-- Begin
-var t = new Array;
+// tooltip positioning constants
+var TMAR = 4;   // top/left margins
+var BMAR = 16;  // bottom/right margins (scrollbars may overhang inner dimensions)
+var XOFF = 10;  // x offset from cursor
+var YOFF = 40;  // y offset
+var YMIN = 10;  // minimum y offset
+var YTOP = 20;  // y offset when above cursor
+// common variables
+var safari1 = navigator.userAgent.indexOf("Safari/312.6") >= 0;
+var ie6 = navigator.userAgent.toLowerCase().indexOf('msie 6') >= 0;
 var mspan = new Array;
+var hlist, tt, tb;
+
 function GetElementsByClass(classname, tagname) {
   var found = new Array();
   var list = document.getElementsByTagName(tagname);
@@ -78,6 +112,53 @@ function GetElementsByClass(classname, tagname) {
   delete list;
   return found;
 }
+
+// move tooltip
+function move(e)
+{
+  if (!tt) return;
+  if (ie6 && (tt.style.top  == '' || tt.style.top  == 0) &&
+             (tt.style.left == '' || tt.style.left == 0))
+  {
+    tt.style.width  = tt.offsetWidth  + 'px';
+    tt.style.height = tt.offsetHeight + 'px';
+  }
+  var w, h;
+  // browser inconsistencies make getting window size more complex than it should be,
+  // and even then we don't know if it is smaller due to scrollbar width
+  if (typeof(window.innerWidth) == 'number') {
+    w = window.innerWidth;
+    h = window.innerHeight;
+  } else if (document.documentElement && document.documentElement.clientWidth) {
+    w = document.documentElement.clientWidth;
+    h = document.documentElement.clientHeight;
+  } else {
+    w = document.body.clientWidth;
+    h = document.body.clientHeight;
+  }
+  var x = e.clientX + XOFF;
+  var y = e.clientY + YOFF;
+  if (safari1) { // patch for people still using OS X 10.3.9
+    x -= document.body.scrollLeft + document.documentElement.scrollLeft;
+    y -= document.body.scrollTop  + document.documentElement.scrollTop;
+  }
+  var mx = w - BMAR - tt.offsetWidth;
+  var my = h - BMAR - tt.offsetHeight;
+  if (y > my + YOFF - YMIN) y = e.clientY - YTOP - tt.offsetHeight;
+  if (x > mx) x = mx;
+  if (y > my) y = my;
+  if (x < TMAR) x = TMAR;
+  if (y < TMAR) y = TMAR;
+  x += document.body.scrollLeft + document.documentElement.scrollLeft;
+  y += document.body.scrollTop  + document.documentElement.scrollTop;
+  tb.style.width  = tt.offsetWidth  + 'px';
+  tb.style.height = tt.offsetHeight + 'px';
+  tt.style.top  = tb.style.top  = y + 'px';
+  tt.style.left = tb.style.left = x + 'px';
+  tt.style.visibility = tb.style.visibility = 'visible';
+}
+
+// highlight/unhighlight text
 function high(e,on) {
   var targ;
   if (e.target) targ = e.target;
@@ -85,32 +166,43 @@ function high(e,on) {
   if (targ.nodeType == 3) targ = targ.parentNode; // defeat Safari bug
   if (!targ.name) targ = targ.parentNode; // go up another level if necessary
   if (targ.name && document.getElementsByName) {
-    var col;
-    var tip;
+    // un-highlight current objects
+    if (hlist) {
+      for (var i=0; i<hlist.length; ++i) {
+        hlist[i].style.background = 'transparent';
+      }
+      hlist = null;
+    }
+    if (tt) {
+      // hide old tooltip
+      tt.style.visibility = tb.style.visibility = 'hidden';
+      tt = null;
+    }
     if (on) {
-      col = "#ffcc99";
       if (targ.name.substring(0,1) == 't') {
-        var index = parseInt(targ.name.substring(1));
-        tip = t[index];
-        if (tip) delete t[index];
+        // show our tooltip (ID is different than name to avoid confusing IE)
+        tt = document.getElementById('p' + targ.name.substring(1));
+        if (tt) {
+          tb = document.getElementById('tb');
+          move(e);
+        }
       }
-    } else {
-      col = "transparent";
-    }
-    // highlight anchor elements with the same name and add tool tip
-    var list = document.getElementsByName(targ.name);
-    for (var i=0; i<list.length; ++i) {
-      list[i].style.background = col;
-      if (tip) list[i].title += tip;
-    }
-    // use class name to highlight span elements if necessary
-    for (var i=0; i<mspan.length; ++i) {
-      if (mspan[i] != targ.name) continue;
-      list = GetElementsByClass(targ.name, 'span');
-      for (var j=0; j<list.length; ++j) {
-        list[j].style.background = col;
+      // highlight anchor elements with the same name
+      hlist = document.getElementsByName(targ.name);
+      // use class name to highlight span elements if necessary
+      for (var i=0; i<mspan.length; ++i) {
+        if (mspan[i] != targ.name) continue;
+        var slist = GetElementsByClass(targ.name, 'span');
+        // add elements from hlist collection to our array
+        for (var j=0; j<hlist.length; ++j) {
+            slist[slist.length] = hlist[j];
+        }
+        hlist = slist;
+        break;
+      }
+      for (var j=0; j<hlist.length; ++j) {
+        hlist[j].style.background = '#ffcc99';
       }
-      break;
     }
   }
 }
@@ -121,10 +213,11 @@ my $htmlHeader3 = q[
 </script></head>
 <body><noscript><b class=V>--&gt;
 Enable JavaScript for active highlighting and information tool tips!
-</b></noscript><table class=dump cellspacing=0 cellpadding=2>
+</b></noscript>
+<table class=dump cellspacing=0 cellpadding=2>
 <tr><td valign='top'><pre>];
 
-my $preMouse = q(<pre onmouseover="high(event,1)" onmouseout="high(event,0)">);
+my $preMouse = q(<pre onmouseover="high(event,1)" onmouseout="high(event,0)" onmousemove="move(event)">);
 
 #------------------------------------------------------------------------------
 # New - create new HtmlDump object
@@ -146,24 +239,29 @@ sub new
 #       0x02 - print red address
 #       0x04 - maker notes data ('M'-class span)
 #       0x08 - limit block length
+#       0x10 - allow double references
+#       0x100 - (reserved)
 # Notes: Block will be shown in 'unused' color if comment string begins with '['
 sub Add($$$$;$$)
 {
     my ($self, $start, $size, $msg, $tip, $flag, $sameTip) = @_;
     my $block = $$self{Block};
     $$block{$start} or $$block{$start} = [ ];
+    my $htip;
     if ($tip and $tip eq 'SAME') {
-        $tip = '';
+        $htip = '';
     } else {
-        $tip = defined $tip ? '\n' . $tip : '';
-        my $m = $msg;
-        $m =~ s/<.*?>//g;       # remove html format codes
-        $tip = "$m$tip";        # add msg as first line in tooltip
+        # use message as first line of tip, and make bold unless in brackets
+        $htip = ($msg =~ /^[[(]/) ? $msg : "<b>$msg</b>";
+        if (defined $tip) {
+            ($tip = EscapeHTML($tip)) =~ s/\n/<br>/g;   # HTML-ize tooltip text
+            $htip .= '<br>' . $tip;
+        }
         # add size if not already done
-        $tip .= "\\n($size bytes)" unless $tip =~ /\\nSize:/;
+        $htip .= "<br>($size bytes)" unless $htip =~ /<br>Size:/;
         ++$self->{TipNum};
     }
-    push @{$$block{$start}}, [ $size, $msg, $tip, $flag, $self->{TipNum} ];
+    push @{$$block{$start}}, [ $size, $msg, $htip, $flag, $self->{TipNum} ];
 }
 
 #------------------------------------------------------------------------------
@@ -172,7 +270,9 @@ sub Add($$$$;$$)
 #         2) data pointer, 3) data position, 4) output file or scalar reference,
 #         5) limit level (1-3), 6) title
 # Returns: non-zero if useful output was generated,
-#          or -1 on error loading data and ERROR is set to offending data name
+#          or -1 on error loading data and "ERROR" is set to offending data name
+# Note: The "Error" member may be set externally to print a specific error
+#       message instead of doing the dump.
 sub Print($$;$$$$$)
 {
     local $_;
@@ -184,7 +284,6 @@ sub Print($$;$$$$$)
     $title = 'HtmlDump' unless $title;
     $level or $level = 0;
     my $tell = $raf->Tell();
-    my @starts = sort { $a <=> $b } keys %$block;
     my $pos = 0;
     my $dataEnd = $dataPos + ($dataPt ? length($$dataPt) : 0);
     # initialize member variables
@@ -208,9 +307,11 @@ sub Print($$;$$$$$)
         $self->{Closed}->[$i] = { ID => [ ], Element => { } };
     }
     $bkgStart = $bkgEnd = 0;
-    $bkgSpan = '';
+    undef @bkgSpan;
     my $index = 0;  # initialize tooltip index
-    my (@names, $wasUnused);
+    my (@names, $wasUnused, @starts);
+    # only do dump if we didn't have a serious error
+    @starts = sort { $a <=> $b } keys %$block unless $$self{Error};
     for ($i=0; $i<@starts; ++$i) {
         my $start = $starts[$i];
         my $parmList = $$block{$start};
@@ -218,10 +319,24 @@ sub Print($$;$$$$$)
         if ($len > 0 and not $wasUnused) {
             # we have an unused bytes before this data block
             --$i;           # dump the data block next time around
-            $start = $pos;  # dump the unused bytes now
+            # split unused data into 2 blocks if it spans end of a bkg block
+            my ($nextBkgEnd, $bkg);
+            if (not defined $wasUnused and $bkgEnd) {
+                foreach $bkg (@bkgSpan) {
+                    next if $pos >= $$bkg{End} + $dataPos or $pos + $len <= $$bkg{End} + $dataPos;
+                    $nextBkgEnd = $$bkg{End} unless $nextBkgEnd and $nextBkgEnd < $$bkg{End};
+                }
+            }
+            if ($nextBkgEnd) {
+                $start = $pos;
+                $len = $nextBkgEnd + $dataPos - $pos;
+                $wasUnused = 0;
+            } else {
+                $start = $pos;  # dump the unused bytes now
+                $wasUnused = 1; # avoid re-dumping unused bytes if we get a read error
+            }
             my $str = ($len > 1) ? "unused $len bytes" : 'pad byte';
             $parmList = [ [ $len, "[$str]", undef, 0x108 ] ];
-            $wasUnused = 1; # avoid re-dumping unused bytes if we get a read error
         } else {
             undef $wasUnused;
         }
@@ -242,10 +357,15 @@ sub Print($$;$$$$$)
                 $names[$tipNum] = $name if defined $tipNum;
                 ++$index;
             }
-            if ($flag == 4) {
-                $bkgStart = $start - $dataPos;
-                $bkgEnd = $bkgStart + $len;
-                $bkgSpan = "<span class='$name M'>";
+            if ($flag & 0x14) {
+                my %bkg = (
+                    Class => $flag & 0x04 ? "$name M" : $name,
+                    Start => $start - $dataPos,
+                    End   => $start - $dataPos + $len,
+                );
+                push @bkgSpan, \%bkg;
+                $bkgStart = $bkg{Start} unless $bkgStart and $bkgStart < $bkg{Start};
+                $bkgEnd = $bkg{End} unless $bkgEnd and $bkgEnd > $bkg{End};
                 push @{$self->{MSpanList}}, $name;
                 next;
             }
@@ -273,7 +393,7 @@ sub Print($$;$$$$$)
                                 # reset $len to the actual length of available data
                                 $raf->Seek(0, 2);
                                 $len = $raf->Tell() - $start;
-                                $tip .= "\\nError: Only $len bytes available!" if $tip;
+                                $tip .= "<br>Error: Only $len bytes available!" if $tip;
                                 next;
                             }
                             $buff .= $buf2;
@@ -281,6 +401,8 @@ sub Print($$;$$$$$)
                         }
                     } else {
                         $err = $msg;
+                        $len = length $buff;
+                        $tip .= "<br>Error: Only $len bytes available!" if $tip;
                     }
                 }
                 last;
@@ -311,11 +433,6 @@ sub Print($$;$$$$$)
         for ($i=0; $i<@$mspan; ++$i) {
             Write($outfile, qq(mspan[$i] = "$$mspan[$i]";\n));
         }
-        my $tips = \@{$$self{TipList}};
-        for ($i=0; $i<@$tips; ++$i) {
-            Write($outfile, qq(t[$i] = "$$tips[$i]";\n)) if defined $$tips[$i];
-        }
-        delete $$self{TipList};
         Write($outfile, $htmlHeader3, $self->{Cols}->[0]);
         Write($outfile, '</pre></td><td valign="top">',
                         $preMouse, $self->{Cols}->[1]);
@@ -323,11 +440,17 @@ sub Print($$;$$$$$)
                         $preMouse, $self->{Cols}->[2]);
         Write($outfile, '</pre></td><td valign="top">',
                         $preMouse, $self->{Cols}->[3]);
-        Write($outfile, "</pre></td></tr></table>\n");
+        Write($outfile, "</pre></td></tr></table>\n<div id=tb class=tb> </div>\n");
+        my $tips = \@{$$self{TipList}};
+        for ($i=0; $i<@$tips; ++$i) {
+            my $tip = $$tips[$i];
+            Write($outfile, "<div id=p$i class=tt>$tip</div>\n") if defined $tip;
+        }
+        delete $$self{TipList};
         $rtnVal = 1;
     } else {
-        Write($outfile, "$title</title></head><body>\n",
-                        "No EXIF or TIFF information found in image\n");
+        my $err = $$self{Error} || 'No EXIF or TIFF information found in image';
+        Write($outfile, "$title</title></head><body>\n$err\n");
         $rtnVal = 0;
     }
     Write($outfile, "</body></html>\n");
@@ -454,7 +577,7 @@ sub DumpTable($$$;$$$$$)
             }
             ++$id unless $dblRef;
         }
-        $name = qq{<a name=$name class=$id>};
+        $name = "<a name=$name class=$id>";
         $msg and $msg = "$name$msg</a>";
     } else {
         $name = '';
@@ -483,7 +606,21 @@ sub DumpTable($$$;$$$$$)
     }
     # loop through each column of hex numbers
     for (;;) {
-        $self->Open('bkg', ($p>=$bkgStart and $p<$bkgEnd) ? $bkgSpan : '', 1, 2);
+        my (@spanClass, @spanCont, $spanClose, $bkg);
+        if ($p >= $bkgStart and $p < $bkgEnd) {
+            foreach $bkg (@bkgSpan) {
+                next unless $p >= $$bkg{Start} and $p < $$bkg{End};
+                push @spanClass, $$bkg{Class};
+                if ($p + 1 == $$bkg{End}) {
+                    $spanClose = 1;
+                } else {
+                    push @spanCont, $$bkg{Class};   # this span continues
+                }
+            }
+            $self->Open('bkg', @spanClass ? "<span class='@spanClass'>" : '', 1, 2);
+        } else {
+            $self->Open('bkg', '', 1, 2);
+        }
         $self->Open('a', $name, 1, 2);
         my $ch = substr($$blockPt,$p-$pos-$skipped,1);
         $c[1] .= sprintf("%.2x", ord($ch));
@@ -496,9 +633,10 @@ sub DumpTable($$$;$$$$$)
         ++$p;
         ++$cols;
         # close necessary elements
-        if ($p >= $bkgEnd) {
+        if ($spanClose) {
+            my $spanCont = @spanCont ? "<span class='@spanCont'>" : '';
             # close without reopening if closing anchor later
-            my $arg = ($p - $pos >= $len) ? 0 : '';
+            my $arg = ($p - $pos >= $len) ? 0 : $spanCont;
             $self->Open('bkg', $arg, 1, 2);
         }
         if ($dblRef and $p >= $endPos) {
@@ -533,7 +671,7 @@ sub DumpTable($$$;$$$$$)
                 my $n = ($len - $p + $pos - $lim) & ~0x0f;
                 if ($n > 16) { # (no use just cutting out one line)
                     $self->Open('bkg', '', 1, 2); # no underline
-                    my $note = "[snip $n bytes]";
+                    my $note = sprintf "[snip %d lines]", $n / 16;
                     $note = (' ' x (24-length($note)/2)) . $note;
                     $c[0] .= "  ...\n";
                     $c[1] .= $note . (' ' x (48-length($note))) . "\n";
@@ -588,20 +726,47 @@ sub FinishTiffDump($$$)
         OtherImageStart   => 'OtherImageLength',
         ImageOffset       => 'ImageByteCount',
         AlphaOffset       => 'AlphaByteCount',
+        MPImageStart      => 'MPImageLength',
+        IDCPreviewStart   => 'IDCPreviewLength',
     );
 
     # add TIFF data to html dump
     foreach $tag (keys %offsetPair) {
         my $info = $exifTool->GetInfo($tag);
         next unless %$info;
-        foreach $key (%$info) {
+        # Panasonic hack: StripOffsets is not valid for Panasonic RW2 files,
+        # and StripRowBytes is not valid for some RAW images
+        if ($tag eq 'StripOffsets' and $exifTool->{TAG_INFO}{$tag}{PanasonicHack}) {
+            # use RawDataOffset instead if available since it is valid in RW2
+            my $info2 = $exifTool->GetInfo('RawDataOffset');
+            $info2 = $info unless %$info2;
+            my @keys = keys %$info2;
+            my $offset = $$info2{$keys[0]};
+            my $raf = $$exifTool{RAF};
+            # ignore StripByteCounts and assume raw data runs to the end of file
+            if (@keys == 1 and $offset =~ /^\d+$/ and $raf) {
+                my $pos = $raf->Tell();
+                $raf->Seek(0, 2);   # seek to end
+                my $len = $raf->Tell() - $offset;
+                $raf->Seek($pos, 0);
+                if ($len > 0) {
+                    $self->Add($offset, $len, "(Panasonic raw data)", "Size: $len bytes", 0x08);
+                    next;
+                }
+            }
+        }
+        # loop through all offsets tags
+        foreach $key (keys %$info) {
             my $name = Image::ExifTool::GetTagName($key);
             my $grp1 = $exifTool->GetGroup($key, 1);
             my $info2 = $exifTool->GetInfo($offsetPair{$tag}, { Group1 => $grp1 });
-            next unless %$info2;
-            my ($key2) = keys %$info2;
+            my $key2 = $offsetPair{$tag};
+            $key2 .= $1 if $key =~ /( .*)/; # use same instance number as $tag
+            next unless $$info2{$key2};
             my $offsets = $$info{$key};
             my $byteCounts = $$info2{$key2};
+            # ignore primary MPImage (this is the whole JPEG)
+            next if $tag eq 'MPImageStart' and $offsets eq '0';
             # (long lists may be SCALAR references)
             my @offsets = split ' ', (ref $offsets ? $$offsets : $offsets);
             my @byteCounts = split ' ', (ref $byteCounts ? $$byteCounts : $byteCounts);
@@ -695,7 +860,7 @@ This module contains code used to generate an HTML-based hex dump of
 information for debugging purposes.  This is code is called when the
 ExifTool 'HtmlDump' option is used.
 
-Currently, only EXIF and TIFF information is dumped.
+Currently, only EXIF/TIFF and JPEG information is dumped.
 
 =head1 BUGS
 
@@ -704,12 +869,11 @@ may run extremely slowly when processing large files with this version of
 Perl.
 
 An HTML 4 compliant browser is needed to properly display the generated HTML
-page, but note that some of these browsers (like Mozilla) may not properly
-display linefeeds in the tool tips.
+page.
 
 =head1 AUTHOR
 
-Copyright 2003-2008, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2010, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
