@@ -11,6 +11,7 @@
 #               2) http://www.faqs.org/docs/png/
 #               3) http://www.libpng.org/pub/mng/
 #               4) http://www.libpng.org/pub/png/spec/register/
+#               5) ftp://ftp.simplesystems.org/pub/png/documents/pngext-1.4.0-pdg.html
 #
 # Notes:        I haven't found a sample PNG image with a 'iTXt' chunk, so
 #               this part of the code is still untested.
@@ -28,7 +29,7 @@ use strict;
 use vars qw($VERSION $AUTOLOAD);
 use Image::ExifTool qw(:DataAccess :Utils);
 
-$VERSION = '1.17';
+$VERSION = '1.22';
 
 sub ProcessPNG_tEXt($$$);
 sub ProcessPNG_iTXt($$$);
@@ -47,6 +48,25 @@ my %pngLookup = (
     "\x8bJNG\r\n\x1a\n" => ['JNG', 'JHDR', 'IEND' ],
 );
 
+# map for directories in PNG images
+my %pngMap = (
+    IFD1         => 'IFD0',
+    EXIF         => 'IFD0', # to write EXIF as a block
+    ExifIFD      => 'IFD0',
+    GPS          => 'IFD0',
+    SubIFD       => 'IFD0',
+    GlobParamIFD => 'IFD0',
+    PrintIM      => 'IFD0',
+    InteropIFD   => 'ExifIFD',
+    MakerNotes   => 'ExifIFD',
+    IFD0         => 'PNG',
+    XMP          => 'PNG',
+    ICC_Profile  => 'PNG',
+    Photoshop    => 'PNG',
+    IPTC         => 'Photoshop',
+    MakerNotes   => 'ExifIFD',
+);
+
 # color type of current image
 $Image::ExifTool::PNG::colorType = -1;
 
@@ -62,6 +82,10 @@ $Image::ExifTool::PNG::colorType = -1;
         Name => 'PrimaryChromaticities',
         SubDirectory => { TagTable => 'Image::ExifTool::PNG::PrimaryChromaticities' },
     },
+    dSIG => {
+        Name => 'DigitalSignature',
+        Binary => 1,
+    },
     fRAc => {
         Name => 'FractalParameters',
         Binary => 1,
@@ -130,6 +154,10 @@ $Image::ExifTool::PNG::colorType = -1;
         Name => 'SignificantBits',
         ValueConv => 'join(" ",unpack("C*",$val))',
     },
+    sCAL => { # png 1.4.0
+        Name => 'SubjectScale',
+        SubDirectory => { TagTable => 'Image::ExifTool::PNG::SubjectScale' },
+    },
     sPLT => {
         Name => 'SuggestedPalette',
         Binary => 1,
@@ -145,6 +173,10 @@ $Image::ExifTool::PNG::colorType = -1;
             3 => 'Absolute Colorimetric',
         },
     },
+    sTER => { # png 1.4.0
+        Name => 'StereoImage',
+        SubDirectory => { TagTable => 'Image::ExifTool::PNG::StereoImage' },
+    },
     tEXt => {
         Name => 'TextualData',
         SubDirectory => { TagTable => 'Image::ExifTool::PNG::TextualData' },
@@ -175,6 +207,10 @@ $Image::ExifTool::PNG::colorType = -1;
         Notes => 'obsolete location specified by older XMP draft',
         SubDirectory => { TagTable => 'Image::ExifTool::XMP::Main' },
     },
+    vpAg => { # private imagemagick chunk
+        Name => 'VirtualPage',
+        SubDirectory => { TagTable => 'Image::ExifTool::PNG::VirtualPage' },
+    },
     zTXt => {
         Name => 'CompressedText',
         SubDirectory => {
@@ -255,6 +291,51 @@ $Image::ExifTool::PNG::colorType = -1;
     },
 );
 
+# PNG sCAL chunk
+%Image::ExifTool::PNG::SubjectScale = (
+    PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
+    GROUPS => { 2 => 'Image' },
+    0 => {
+        Name => 'SubjectUnits',
+        PrintConv => { 1 => 'Meters', 2 => 'Radians' },
+    },
+    1 => {
+        Name => 'SubjectPixelWidth',
+        Format => 'var_string',
+    },
+    2 => {
+        Name => 'SubjectPixelHeight',
+        Format => 'var_string',
+    },
+);
+
+# PNG vpAg chunk
+%Image::ExifTool::PNG::VirtualPage = (
+    PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
+    GROUPS => { 2 => 'Image' },
+    FORMAT => 'int32u',
+    0 => 'VirtualImageWidth',
+    1 => 'VirtualImageHeight',
+    2 => {
+        Name => 'VirtualPageUnits',
+        Format => 'int8u',
+        # what is the conversion for this?
+    },
+);
+
+# PNG sTER chunk
+%Image::ExifTool::PNG::StereoImage = (
+    PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
+    GROUPS => { 2 => 'Image' },
+    0 => {
+        Name => 'StereoMode',
+        PrintConv => {
+            0 => 'Cross-fuse Layout',
+            1 => 'Diverging-fuse Layout',
+        },
+    },
+);
+
 my %unreg = ( Notes => 'unregistered' );
 
 # Tags for PNG tEXt zTXt and iTXt chunks
@@ -266,21 +347,21 @@ my %unreg = ( Notes => 'unregistered' );
     PREFERRED => 1, # always add these tags when writing
     GROUPS => { 2 => 'Image' },
     NOTES => q{
-The PNG TextualData format allows aribrary tag names to be used.  The tags
-listed below are the only ones that can be written (unless new user-defined
-tags are added via the configuration file), however ExifTool will extract
-any other TextualData tags that are found.
-
-Data for the TextualData tags may be stored as tEXt, zTXt or iTXt chunks in
-the PNG image.  ExifTool will read and edit tags in their original form, but
-new string tags are created as uncompressed tEXt by default, or as
-compressed zTXt if the -z (Compress) option is used and Compress::Zlib is
-available. Raw profile information is always created as compressed zTXt if
-Compress::Zlib is available.
-
-Some of the tags below are not registered as part of the PNG specification,
-but are included here because they are generated by other software such as
-ImageMagick.
+        The PNG TextualData format allows aribrary tag names to be used.  The tags
+        listed below are the only ones that can be written (unless new user-defined
+        tags are added via the configuration file), however ExifTool will extract
+        any other TextualData tags that are found.
+        
+        Data for the TextualData tags may be stored as tEXt, zTXt or iTXt chunks in
+        the PNG image.  ExifTool will read and edit tags in their original form, but
+        new string tags are created as uncompressed tEXt by default, or as
+        compressed zTXt if the -z (Compress) option is used and Compress::Zlib is
+        available.  Raw profile information is always created as compressed zTXt if
+        Compress::Zlib is available, and standard XMP is written as iTXt.
+        
+        Some of the tags below are not registered as part of the PNG specification,
+        but are included here because they are generated by other software such as
+        ImageMagick.
     },
     Title       => { },
     Author      => { Groups => { 2 => 'Author' } },
@@ -306,13 +387,34 @@ ImageMagick.
     Label       => { %unreg },
     Make        => { %unreg, Groups => { 2 => 'Camera' } },
     Model       => { %unreg, Groups => { 2 => 'Camera' } },
+   'create-date'=> {
+        Name => 'CreateDate',
+        Groups => { 2 => 'Time' },
+        Shift => 'Time',
+        %unreg,
+        ValueConv => 'require Image::ExifTool::XMP; Image::ExifTool::XMP::ConvertXMPDate($val)',
+        ValueConvInv => 'require Image::ExifTool::XMP; Image::ExifTool::XMP::FormatXMPDate($val)',
+        PrintConv => '$self->ConvertDateTime($val)',
+        PrintConvInv => '$self->InverseDateTime($val,undef,1)',
+    },
+   'modify-date'=> {
+        Name => 'ModDate', # (to distinguish from tIME chunk "ModifyDate")
+        Groups => { 2 => 'Time' },
+        Shift => 'Time',
+        %unreg,
+        ValueConv => 'require Image::ExifTool::XMP; Image::ExifTool::XMP::ConvertXMPDate($val)',
+        ValueConvInv => 'require Image::ExifTool::XMP; Image::ExifTool::XMP::FormatXMPDate($val)',
+        PrintConv => '$self->ConvertDateTime($val)',
+        PrintConvInv => '$self->InverseDateTime($val,undef,1)',
+    },
     TimeStamp   => { %unreg, Groups => { 2 => 'Time' }, Shift => 'Time' },
     URL         => { %unreg },
    'XML:com.adobe.xmp' => {
         Name => 'XMP',
         Notes => q{
-            location according to the XMP specification -- this is where ExifTool will
-            add a new XMP chunk if the image didn't already contain XMP
+            unregistered, but this is the location according to the XMP specification,
+            and is where ExifTool will add a new XMP chunk if the image didn't already
+            contain XMP
         },
         SubDirectory => {
             TagTable => 'Image::ExifTool::XMP::Main',
@@ -444,9 +546,7 @@ sub FoundPNG($$$$;$$)
                     my $name = $tagName;
                     $wasCompressed and $name = "Decompressed $name";
                     $exifTool->VerboseDir($name, 0, $len);
-                    my %parms = ( Prefix => $exifTool->{INDENT}, Out => $out );
-                    $parms{MaxLen} = 96 unless $verbose > 3;
-                    Image::ExifTool::HexDump(\$val, undef, %parms);
+                    $exifTool->VerboseDump(\$val);
                 }
                 # don't indent next directory (since it is really the same data)
                 $exifTool->{INDENT} =~ s/..$//;
@@ -495,10 +595,10 @@ sub FoundPNG($$$$;$$)
                     delete $exifTool->{ADD_PNG}->{$tag};
                     # (also handle case of tEXt tags written with lowercase first letter)
                     delete $exifTool->{ADD_PNG}->{ucfirst($tag)};
-                    my $newValueHash = $exifTool->GetNewValueHash($tagInfo);
-                    $isOverwriting = Image::ExifTool::IsOverwriting($newValueHash);
+                    my $nvHash = $exifTool->GetNewValueHash($tagInfo);
+                    $isOverwriting = Image::ExifTool::IsOverwriting($nvHash);
                     if (defined $deflateErr) {
-                        $newVal = Image::ExifTool::GetNewValues($newValueHash);
+                        $newVal = Image::ExifTool::GetNewValues($nvHash);
                         # can only write tag now if unconditionally deleting it
                         if ($isOverwriting > 0 and not defined $newVal) {
                             $val = '<deflate error>';
@@ -508,19 +608,17 @@ sub FoundPNG($$$$;$$)
                         }
                     } else {
                         if ($isOverwriting < 0) {
-                            $isOverwriting = Image::ExifTool::IsOverwriting($newValueHash, $val);
+                            $isOverwriting = Image::ExifTool::IsOverwriting($nvHash, $val);
                         }
                         # (must get new value after IsOverwriting() in case it was shifted)
-                        $newVal = Image::ExifTool::GetNewValues($newValueHash);
+                        $newVal = Image::ExifTool::GetNewValues($nvHash);
                     }
                 }
                 if ($isOverwriting) {
                     $$outBuff =  (defined $newVal) ? $newVal : '';
                     ++$exifTool->{CHANGED};
-                    if ($verbose > 1) {
-                        print $out "    - PNG:$tagName = '",$exifTool->Printable($val),"'\n";
-                        print $out "    + PNG:$tagName = '",$exifTool->Printable($newVal),"'\n" if defined $newVal;
-                    }
+                    $exifTool->VerboseValue("- PNG:$tagName", $val);
+                    $exifTool->VerboseValue("+ PNG:$tagName", $newVal) if defined $newVal;
                 }
             }
             if ($$outBuff) {
@@ -605,12 +703,7 @@ sub ProcessProfile($$$)
     if ($verbose) {
         if ($verbose > 2) {
             $exifTool->VerboseDir("Decoded $tagName", 0, $len);
-            my %parms = (
-                Prefix => $exifTool->{INDENT},
-                Out => $exifTool->Options('TextOut'),
-            );
-            $parms{MaxLen} = 96 unless $verbose > 3;
-            Image::ExifTool::HexDump(\$buff, undef, %parms);
+            $exifTool->VerboseDump(\$buff);
         }
         # don't indent next directory (since it is really the same data)
         $exifTool->{INDENT} =~ s/..$//;
@@ -647,14 +740,13 @@ sub ProcessProfile($$$)
         my $hdrLen = length($Image::ExifTool::exifAPP1hdr);
         $dirInfo{DirStart} += $hdrLen;
         $dirInfo{DirLen} -= $hdrLen;
-        $processed = $exifTool->ProcessTIFF(\%dirInfo);
         if ($outBuff) {
-            if ($$outBuff) {
-                $$outBuff = $Image::ExifTool::exifAPP1hdr . $$outBuff if $$outBuff;
-            } else {
-                $$outBuff = '' if $processed;
-            }
+            $$outBuff = $exifTool->WriteDirectory(\%dirInfo, $tagTablePtr,
+                                                  \&Image::ExifTool::WriteTIFF);
+            $$outBuff = $Image::ExifTool::exifAPP1hdr . $$outBuff if $$outBuff;
             delete $$addDirs{IFD0};
+        } else {
+            $processed = $exifTool->ProcessTIFF(\%dirInfo);
         }
     } elsif ($buff =~ /^$Image::ExifTool::xmpAPP1hdr/) {
         # APP1 XMP information
@@ -673,14 +765,13 @@ sub ProcessProfile($$$)
     } elsif ($buff =~ /^(MM\0\x2a|II\x2a\0)/) {
         # TIFF information (haven't seen this, but what the heck...)
         return 1 if $outBuff and not $$editDirs{IFD0};
-        $processed = $exifTool->ProcessTIFF(\%dirInfo);
         if ($outBuff) {
-            if ($$outBuff) {
-                $$outBuff = $Image::ExifTool::exifAPP1hdr . $$outBuff if $$outBuff;
-            } else {
-                $$outBuff = '' if $processed;
-            }
+            $$outBuff = $exifTool->WriteDirectory(\%dirInfo, $tagTablePtr,
+                                                  \&Image::ExifTool::WriteTIFF);
+            $$outBuff = $Image::ExifTool::exifAPP1hdr . $$outBuff if $$outBuff;
             delete $$addDirs{IFD0};
+        } else {
+            $processed = $exifTool->ProcessTIFF(\%dirInfo);
         }
     } else {
         my $profName = $profileType;
@@ -782,8 +873,8 @@ sub ProcessPNG($$)
         $exifTool->{ADD_PNG} = $exifTool->GetNewTagInfoHash(
             \%Image::ExifTool::PNG::Main,
             \%Image::ExifTool::PNG::TextualData);
-        # initialize with same directories as JPEG, but PNG tags take priority
-        $exifTool->InitWriteDirs('JPEG','PNG');
+        # initialize with same directories, with PNG tags taking priority
+        $exifTool->InitWriteDirs(\%pngMap,'PNG');
     }
     my ($fileType, $hdrChunk, $endChunk) = @{$pngLookup{$sig}};
     $exifTool->SetFileType($fileType);  # set the FileType tag
@@ -864,11 +955,7 @@ sub ProcessPNG($$)
                 next;
             }
             print $out "$fileType $chunk ($len bytes):\n";
-            if ($verbose > 2) {
-                my %dumpParms = ( Out => $out, Addr => $raf->Tell() - $len - 4 );
-                $dumpParms{MaxLen} = 96 if $verbose <= 4;
-                Image::ExifTool::HexDump(\$dbuf, undef, %dumpParms);
-            }
+            $exifTool->VerboseDump(\$dbuf, Addr => $raf->Tell() - $len - 4) if $verbose > 2;
         }
         # only extract information from chunks in our tables
         my ($theBuff, $outBuff);
@@ -915,7 +1002,7 @@ and JNG (JPEG Network Graphics) images.
 
 =head1 AUTHOR
 
-Copyright 2003-2008, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2010, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
