@@ -39,7 +39,7 @@ my %manufacturer = (
     12 => 'Visualize, Spain',
 );
 
-my %iptcCharsetInv = ( 'UTF8' => "\x1b%G" );
+my %iptcCharsetInv = ( 'UTF8' => "\x1b%G", 'UTF-8' => "\x1b%G" );
 
 # ISO 2022 Character Coding Notes
 # -------------------------------
@@ -70,7 +70,7 @@ my %iptcCharsetInv = ( 'UTF8' => "\x1b%G" );
 # Control chars (designated and invoked)
 #   C0 : 0x1b 0x21 F (0x21 = '!')
 #   C1 : 0x1b 0x22 F (0x22 = '"')
-# Complete codes (control+graphics, designated and involked)
+# Complete codes (control+graphics, designated and invoked)
 #   0x1b 0x25 F   (0x25 = '%')
 #   0x1b 0x25 I F
 #   0x1b 0x25 0x47 ("\x1b%G") - UTF-8
@@ -97,8 +97,7 @@ sub PrintInvCodedCharset($)
 
 #------------------------------------------------------------------------------
 # validate raw values for writing
-# Inputs: 0) ExifTool object reference, 1) tagInfo hash reference,
-#         2) raw value reference
+# Inputs: 0) ExifTool object ref, 1) tagInfo hash ref, 2) raw value ref
 # Returns: error string or undef (and possibly changes value) on success
 sub CheckIPTC($$$)
 {
@@ -121,14 +120,20 @@ sub CheckIPTC($$$)
         my ($fmt, $minlen, $maxlen) = ($1, $2, $3);
         my $len = length $$valPtr;
         if ($fmt eq 'digits') {
-            return 'Non-numeric characters in value' unless $$valPtr =~ /^\d+$/;
-            # left pad with zeros if necessary
-            $$valPtr = ('0' x ($len - $minlen)) . $$valPtr if $len < $minlen;
+            return 'Non-numeric characters in value' unless $$valPtr =~ /^\d*$/;
+            if ($len < $minlen and $len) {
+                # left pad with zeros if necessary
+                $$valPtr = ('0' x ($minlen - $len)) . $$valPtr;
+                $len = $minlen;
+            }
         }
-        if ($minlen) {
+        if (defined $minlen) {
             $maxlen or $maxlen = $minlen;
             return "String too short (minlen is $minlen)" if $len < $minlen;
-            return "String too long (maxlen is $maxlen)" if $len > $maxlen;
+            if ($len > $maxlen and not $exifTool->Options('IgnoreMinorErrors')) {
+                $$exifTool{CHECK_WARN} = "[minor] IPTC:$$tagInfo{Name} exceeds length limit (truncated)";
+                $$valPtr = substr($$valPtr, 0, $maxlen);
+            }
         }
     } else {
         return "Bad IPTC Format ($format)";
@@ -145,8 +150,9 @@ sub CheckIPTC($$$)
 sub FormatIPTC($$$$$;$)
 {
     my ($exifTool, $tagInfo, $valPtr, $xlatPtr, $rec, $read) = @_;
-    return unless $$tagInfo{Format};
-    if ($$tagInfo{Format} =~ /^int(\d+)/) {
+    my $format = $$tagInfo{Format} || $$tagInfo{Table}{FORMAT};
+    return unless $format;
+    if ($format =~ /^int(\d+)/) {
         if ($read) {
             my $len = length($$valPtr);
             if ($len <= 8) {    # limit integer conversion to 8 bytes long
@@ -167,7 +173,7 @@ sub FormatIPTC($$$$$;$)
                 $$valPtr = pack('N', $$valPtr);
             }
         }
-    } elsif ($$tagInfo{Format} =~ /^string/) {
+    } elsif ($format =~ /^string/) {
         if ($rec == 1) {
             if ($$tagInfo{Name} eq 'CodedCharacterSet') {
                 $$xlatPtr = HandleCodedCharset($exifTool, $$valPtr);
@@ -181,30 +187,43 @@ sub FormatIPTC($$$$$;$)
 #------------------------------------------------------------------------------
 # generate IPTC-format date
 # Inputs: 0) EXIF-format date string (YYYY:MM:DD) or date/time string
-# Returns: IPTC-format date string (YYYYMMDD), or undef on error
+# Returns: IPTC-format date string (YYYYMMDD), or undef and issue warning on error
 sub IptcDate($)
 {
     my $val = shift;
-    $val =~ s/.*(\d{4}):?(\d{2}):?(\d{2}).*/$1$2$3/ or undef $val;
+    unless ($val =~ s/.*(\d{4}):?(\d{2}):?(\d{2}).*/$1$2$3/) {
+        warn "Invalid date format (use YYYY:MM:DD)\n";
+        undef $val;
+    }
     return $val;
 }
 
 #------------------------------------------------------------------------------
 # generate IPTC-format time
 # Inputs: 0) EXIF-format time string (HH:MM:SS[+/-HH:MM]) or date/time string
-# Returns: IPTC-format time string (HHMMSS+HHMM), or undef on error
+# Returns: IPTC-format time string (HHMMSS+HHMM), or undef and issue warning on error
 sub IptcTime($)
 {
     my $val = shift;
-    if ($val =~ /\s*\b(\d{1,2}):?(\d{1,2}):?(\d{1,2})(\S*)\s*$/) {
-        $val = sprintf("%.2d%.2d%.2d",$1,$2,$3);
-        my $tz = $4;
-        if ($tz =~ /([+-]\d{1,2}):?(\d{1,2})/) {
-            $val .= sprintf("%+.2d%.2d",$1,$2);
+    if ($val =~ /\s*\b(\d{1,2})(:?)(\d{2})(:?)(\d{2})(\S*)\s*$/ and ($2 or not $4)) {
+        $val = sprintf("%.2d%.2d%.2d",$1,$3,$5);
+        my $tz = $6;
+        if ($tz =~ /([+-]\d{1,2}):?(\d{2})/) {
+            $tz = sprintf("%+.2d%.2d",$1,$2);
+        } elsif ($tz !~ /Z/i) {
+            # use local system timezone by default (note: it is difficult to use
+            # the proper local timezone for this date/time because the date tag
+            # is written separately so we don't know what the local timezone offset
+            # really should be for this date/time)
+            my $now = time;
+            my @tm = localtime($now);
+            ($tz = Image::ExifTool::TimeZoneString(\@tm, $now)) =~ tr/://d;
         } else {
-            $val .= '+0000';    # don't know the time zone
+            $tz = '+0000';  # don't know the time zone
         }
+        $val .= $tz;
     } else {
+        warn "Invalid time format (use HH:MM:SS[+/-HH:MM])\n";
         undef $val;     # time format error
     }
     return $val;
@@ -256,14 +275,23 @@ sub InvConvertPictureNumber($)
 
 #------------------------------------------------------------------------------
 # Write IPTC data record
-# Inputs: 0) ExifTool object reference, 1) source dirInfo reference,
-#         2) tag table reference
+# Inputs: 0) ExifTool object ref, 1) source dirInfo ref, 2) tag table ref
 # Returns: IPTC data block (may be empty if no IPTC data)
 # Notes: Increments ExifTool CHANGED flag for each tag changed
-sub WriteIPTC($$$)
+sub DoWriteIPTC($$$)
 {
     my ($exifTool, $dirInfo, $tagTablePtr) = @_;
-    $exifTool or return 1;    # allow dummy access to autoload this package
+    my $verbose = $exifTool->Options('Verbose');
+    my $out = $exifTool->Options('TextOut');
+
+    # avoid editing IPTC directory unless necessary:
+    # - improves speed
+    # - avoids changing current MD5 digest unnecessarily
+    # - avoids adding mandatory tags unless some other IPTC is changed
+    unless (exists $exifTool->{EDIT_DIRS}->{$$dirInfo{DirName}}) {
+        print $out "$$exifTool{INDENT}  [nothing changed]\n" if $verbose;
+        return undef;
+    }
     my $dataPt = $$dirInfo{DataPt};
     unless ($dataPt) {
         my $emptyData = '';
@@ -271,13 +299,11 @@ sub WriteIPTC($$$)
     }
     my $start = $$dirInfo{DirStart} || 0;
     my $dirLen = $$dirInfo{DirLen};
-    my $verbose = $exifTool->Options('Verbose');
-    my $out = $exifTool->Options('TextOut');
     my ($tagInfo, %iptcInfo, $tag);
 
-    # begin by assuming IPTC is coded in Latin unless otherwise specified
-    my $xlat = $exifTool->Options('Charset');
-    undef $xlat if $xlat eq 'Latin';
+    # start by assuming default IPTC encoding
+    my $xlat = $exifTool->Options('CharsetIPTC');
+    undef $xlat if $xlat eq $exifTool->Options('Charset');
 
     # make sure our dataLen is defined (note: allow zero length directory)
     unless (defined $dirLen) {
@@ -303,7 +329,6 @@ sub WriteIPTC($$$)
         my $table = $tagInfo->{SubDirectory}->{TagTable} or next;
         my $subTablePtr = Image::ExifTool::GetTagTable($table);
         $recordNum{$subTablePtr} = $tag;
-        Image::ExifTool::GenerateTagIDs($subTablePtr);
     }
 
     # loop through new values and accumulate all IPTC information
@@ -350,7 +375,14 @@ sub WriteIPTC($$$)
             ($id, $rec, $tag, $len) = unpack("CCCn", $buff);
             if ($id == 0x1c) {
                 if ($rec < $lastRec) {
-                    return undef if $exifTool->Warn("IPTC doesn't conform to spec: Records out of sequence", 1);
+                    if ($rec == 0) {
+                        return undef if $exifTool->Warn("IPTC record 0 encountered, subsequent records ignored", 1);
+                        undef $rec;
+                        $pos = $dirEnd;
+                        $len = 0;
+                    } else {
+                        return undef if $exifTool->Warn("IPTC doesn't conform to spec: Records out of sequence", 1);
+                    }
                 }
                 # handle extended IPTC entry if necessary
                 $pos += 5;      # step to after field header
@@ -411,7 +443,7 @@ sub WriteIPTC($$$)
                                 }
                                 $tagInfo = $subTablePtr->{$mandTag} or warn("WriteIPTC: Internal error 2\n"), next;
                                 my $value = $mandatory->{$mandTag};
-                                $verbose > 1 and print $out "    + IPTC:$$tagInfo{Name} = '$value' (mandatory)\n";
+                                $exifTool->VerboseValue("+ IPTC:$$tagInfo{Name}", $value, ' (mandatory)');
                                 # apply necessary format conversions
                                 FormatIPTC($exifTool, $tagInfo, \$value, \$xlat, $lastRec);
                                 $len = length $value;
@@ -433,7 +465,7 @@ sub WriteIPTC($$$)
                 # (write out our tags in numerical order even though
                 # this isn't required by the IPTC spec)
                 last if defined $rec and $rec <= $newRec;
-                my $newValueHash = $exifTool->GetNewValueHash($tagInfo);
+                my $nvHash = $exifTool->GetNewValueHash($tagInfo);
                 # only add new values if...
                 my ($doSet, @values);
                 my $found = $foundRec{$newRec}->{$newTag} || 0;
@@ -442,18 +474,18 @@ sub WriteIPTC($$$)
                     $doSet = 1;
                 } elsif ($$tagInfo{List}) {
                     # ...tag is List and it existed before or we are creating it
-                    $doSet = 1 if $found or Image::ExifTool::IsCreating($newValueHash);
+                    $doSet = 1 if $found or Image::ExifTool::IsCreating($nvHash);
                 } else {
                     # ...tag didn't exist before and we are creating it
-                    $doSet = 1 if not $found and Image::ExifTool::IsCreating($newValueHash);
+                    $doSet = 1 if not $found and Image::ExifTool::IsCreating($nvHash);
                 }
                 if ($doSet) {
-                    @values = Image::ExifTool::GetNewValues($newValueHash);
+                    @values = Image::ExifTool::GetNewValues($nvHash);
                     @values and $foundRec{$newRec}->{$newTag} = $found | 0x04;
                     # write tags for each value in list
                     my $value;
                     foreach $value (@values) {
-                        $verbose > 1 and print $out "    + IPTC:$$tagInfo{Name} = '$value'\n";
+                        $exifTool->VerboseValue("+ IPTC:$$tagInfo{Name}", $value);
                         # reset allMandatory flag if a non-mandatory tag is written
                         if ($allMandatory) {
                             my $mandatory = $mandatory{$newRec};
@@ -491,14 +523,14 @@ sub WriteIPTC($$$)
         # write out this record unless we are setting it with a new value
         $tagInfo = $set{$rec}->{$tag};
         if ($tagInfo) {
-            my $newValueHash = $exifTool->GetNewValueHash($tagInfo);
+            my $nvHash = $exifTool->GetNewValueHash($tagInfo);
             $len = $pos - $valuePtr;
             my $val = substr($$dataPt, $valuePtr, $len);
             my $oldXlat = $xlat;
             FormatIPTC($exifTool, $tagInfo, \$val, \$xlat, $rec, 1);
-            if (Image::ExifTool::IsOverwriting($newValueHash, $val)) {
+            if (Image::ExifTool::IsOverwriting($nvHash, $val)) {
                 $xlat = $oldXlat;   # don't change translation (not writing this value)
-                $verbose > 1 and print $out "    - IPTC:$$tagInfo{Name} = '$val'\n";
+                $exifTool->VerboseValue("- IPTC:$$tagInfo{Name}", $val);
                 ++$exifTool->{CHANGED};
                 # set deleted flag to indicate we found and deleted this tag
                 $foundRec{$rec}->{$tag} |= 0x02;
@@ -531,6 +563,63 @@ sub WriteIPTC($$$)
     return $newData;
 }
 
+#------------------------------------------------------------------------------
+# Write IPTC data record and calculate NewIPTCDigest
+# Inputs: 0) ExifTool object ref, 1) source dirInfo ref, 2) tag table ref
+# Returns: IPTC data block (may be empty if no IPTC data)
+# Notes: Increments ExifTool CHANGED flag for each tag changed
+sub WriteIPTC($$$)
+{
+    my ($exifTool, $dirInfo, $tagTablePtr) = @_;
+    $exifTool or return 1;    # allow dummy access to autoload this package
+
+    my $newData = DoWriteIPTC($exifTool, $dirInfo, $tagTablePtr);
+
+    # calculate IPTC digests only if we are writing or deleting
+    # Photoshop:IPTCDigest with a value of 'new' or 'old'
+    while ($Image::ExifTool::Photoshop::iptcDigestInfo) {
+        my $nvHash = $exifTool->{NEW_VALUE}{$Image::ExifTool::Photoshop::iptcDigestInfo};
+        last unless defined $nvHash;
+        my @values = Image::ExifTool::GetNewValues($nvHash);
+        push @values, @{$$nvHash{DelValue}} if $$nvHash{DelValue};
+        my $new = grep /^new$/, @values;
+        my $old = grep /^old$/, @values;
+        last unless $new or $old;
+        unless (eval 'require Digest::MD5') {
+            $exifTool->Warn('Digest::MD5 must be installed to calculate IPTC digest');
+            last;
+        }
+        my $dataPt;
+        if ($new) {
+            if (defined $newData) {
+                $dataPt = \$newData;
+            } else {
+                $dataPt = $$dirInfo{DataPt};
+                if ($$dirInfo{DirStart} or length($$dataPt) != $$dirInfo{DirLen}) {
+                    my $buff = substr($$dataPt, $$dirInfo{DirStart}, $$dirInfo{DirLen});
+                    $dataPt = \$buff;
+                }
+            }
+            # set NewIPTCDigest data member unless IPTC is being deleted
+            $$exifTool{NewIPTCDigest} = Digest::MD5::md5($$dataPt) if length $$dataPt;
+        }
+        if ($old) {
+            if ($new and not defined $newData) {
+                $$exifTool{OldIPTCDigest} = $$exifTool{NewIPTCDigest};
+            } elsif ($$dirInfo{DataPt}) { #(may be undef if creating new IPTC)
+                $dataPt = $$dirInfo{DataPt};
+                if ($$dirInfo{DirStart} or length($$dataPt) != $$dirInfo{DirLen}) {
+                    my $buff = substr($$dataPt, $$dirInfo{DirStart}, $$dirInfo{DirLen});
+                    $dataPt = \$buff;
+                }
+                $$exifTool{OldIPTCDigest} = Digest::MD5::md5($$dataPt) if length $$dataPt;
+            }
+        }
+        last;
+    }
+    return $newData;
+}
+
 
 1; # end
 
@@ -551,7 +640,7 @@ seldom-used routines.
 
 =head1 AUTHOR
 
-Copyright 2003-2007, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2010, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
